@@ -1499,7 +1499,8 @@ on_snapshot_ready (WebKitWebView *webview,
   gtk_tree_model_get_iter (model, &iter, path);
   gtk_tree_path_free (path);
 
-  ephy_overview_store_set_snapshot (EPHY_OVERVIEW_STORE (model), &iter, surface);
+  ephy_overview_store_set_snapshot (EPHY_OVERVIEW_STORE (model), &iter, surface,
+                                    webkit_web_view_get_favicon (webview));
   cairo_surface_destroy (surface);
 }
 
@@ -2057,7 +2058,7 @@ ephy_web_view_normalize_or_autosearch_url (EphyWebView *view, const char *url)
     if (url_search == NULL || url_search[0] == '\0') {
       g_free (url_search);
 
-      url_search = g_strdup (_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8"));
+      url_search = g_strdup (_("http://duckduckgo.com/?q=%s&amp;t=epiphany"));
     }
 
     query_param = soup_form_encode ("q", url, NULL);
@@ -2544,6 +2545,26 @@ ephy_web_view_set_typed_address (EphyWebView *view,
   g_object_notify (G_OBJECT (view), "typed-address");
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+has_modified_forms_cb (GDBusProxy *web_extension,
+                       GAsyncResult *result,
+                       GTask *task)
+{
+  GVariant *return_value;
+  gboolean retval = FALSE;
+
+  return_value = g_dbus_proxy_call_finish (web_extension, result, NULL);
+  if (return_value) {
+    g_variant_get (return_value, "(b)", &retval);
+    g_variant_unref (return_value);
+  }
+
+  g_task_return_boolean (task, retval);
+  g_object_unref (task);
+}
+#endif
+
 /**
  * ephy_web_view_has_modified_forms:
  * @view: an #EphyWebView
@@ -2558,33 +2579,46 @@ ephy_web_view_set_typed_address (EphyWebView *view,
  *
  * Return value: %TRUE if @view has user-modified forms
  **/
-gboolean
-ephy_web_view_has_modified_forms (EphyWebView *view)
+void
+ephy_web_view_has_modified_forms (EphyWebView *view,
+                                  GCancellable *cancellable,
+                                  GAsyncReadyCallback callback,
+                                  gpointer user_data)
 {
-  return FALSE;
-#if 0
+  GTask *task = g_task_new (view, cancellable, callback, user_data);
+#ifdef HAVE_WEBKIT2
   GDBusProxy *web_extension;
-  GVariant *result;
-  gboolean retval = FALSE;
 
-  /* FIXME: This should be async */
   web_extension = ephy_embed_shell_get_web_extension_proxy (ephy_embed_shell_get_default ());
-  if (!web_extension)
-    return FALSE;
-  result = g_dbus_proxy_call_sync (web_extension,
-                                   "HasModifiedForms",
-                                   g_variant_new ("(t)", webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view))),
-                                   G_DBUS_CALL_FLAGS_NONE,
-                                   -1,
-                                   NULL,
-                                   NULL);
+  if (web_extension) {
+    g_dbus_proxy_call (web_extension,
+                       "HasModifiedForms",
+                       g_variant_new ("(t)", webkit_web_view_get_page_id (WEBKIT_WEB_VIEW (view))),
+                       G_DBUS_CALL_FLAGS_NONE,
+                       -1,
+                       cancellable,
+                       (GAsyncReadyCallback)has_modified_forms_cb,
+                       g_object_ref (task));
+  } else {
+    g_task_return_boolean (task, FALSE);
+  }
+#else
+  WebKitDOMDocument *document = webkit_web_view_get_dom_document (WEBKIT_WEB_VIEW (view));
 
-
-  g_variant_get (result, "(b)", &retval);
-  g_variant_unref (result);
-
-  return retval;
+  g_task_return_boolean (task, ephy_web_dom_utils_has_modified_forms (document));
 #endif
+
+  g_object_unref (task);
+}
+
+gboolean
+ephy_web_view_has_modified_forms_finish (EphyWebView *view,
+                                         GAsyncResult *result,
+                                         GError **error)
+{
+  g_return_val_if_fail (g_task_is_valid (result, view), FALSE);
+
+  return g_task_propagate_boolean (G_TASK (result), error);
 }
 
 /**
@@ -2624,7 +2658,7 @@ ephy_web_view_print_failed (EphyWebView *view, GError *error)
   GtkContainer *content_area;
   EphyEmbed *embed = EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (view);
 
-  info_bar = gtk_info_bar_new_with_buttons (GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+  info_bar = gtk_info_bar_new_with_buttons (_("_OK"), GTK_RESPONSE_OK, NULL);
   label = gtk_label_new (error->message);
   content_area = GTK_CONTAINER (gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar)));
 
